from copy import deepcopy
from django.db import models
from django.db.models import Q, Count, Sum, Avg, Max, Min, StdDev, Variance
from django.db.models.query import REPR_OUTPUT_SIZE
from django.db.models.sql.constants import LOOKUP_SEP
from modeltree.utils import M


class Aggregator(object):
    def __init__(self, field, model=None):
        if not isinstance(field, models.Field):
            if not model:
                raise TypeError('Field instance or field name and model class required')
            field_name = field
            field = model._meta.get_field_by_name(field_name)[0]
        else:
            model = field.model
            field_name = field.name

        self.field = field
        self.field_name = field_name
        self.model = model

        self._queryset = None
        self._aggregates = {}
        self._filter = []
        self._exclude = []
        self._having = []
        self._groupby = []
        self._orderby = []

    def __eq__(self, other):
        return list(self) == other

    def __deepcopy__(self):
        return self._clone()

    def __len__(self):
        # If the result cache is filled, use the length otherwise
        # performa databse hit
        if hasattr(self, '_length'):
            return self._length
        return len(self._construct())

    def __repr__(self):
        data = list(self[:REPR_OUTPUT_SIZE + 1])
        if len(data) > REPR_OUTPUT_SIZE:
            data[-1] = '...(remaining elements results)...'
        return repr(data)

    def __getitem__(self, key):
        return list(self._result_iter())[key]

    def __iter__(self):
        return self._result_iter()

    def _result_iter(self):
        if hasattr(self, '_result_cache'):
            for obj in self._result_cache:
                yield obj
        else:
            queryset = self._construct()
            cache = []
            length = 0

            for obj in iter(queryset):
                if self._groupby:
                    keys = []
                    for key in self._groupby:
                        keys.append(obj[key])
                        del obj[key]
                    obj['values'] = keys

                length += 1
                cache.append(obj)
                yield obj

            self._result_cache = cache
            self._length = length

    def _construct(self):
        if self._queryset is None:
            queryset = self.model.objects.all()
        else:
            queryset = self._queryset
        if self._filter:
            queryset = queryset.filter(*self._filter)
        if self._exclude:
            queryset = queryset.exclude(*self._exclude)
        if self._groupby:
            queryset = queryset.values(*self._groupby)
        if self._aggregates:
            if self._groupby:
                queryset = queryset.annotate(**self._aggregates)
            else:
                queryset = queryset.aggregate(**self._aggregates)
        if self._having:
            queryset = queryset.filter(*self._having)
        if self._orderby:
            queryset = queryset.order_by(*self._orderby)
        if not self._groupby:
            return [dict(queryset)]
        return queryset

    def _clone(self):
        clone = self.__class__(self.field_name, self.model)
        clone._aggregates = deepcopy(self._aggregates)
        clone._filter = deepcopy(self._filter)
        clone._exclude = deepcopy(self._exclude)
        clone._having = deepcopy(self._having)
        clone._groupby = deepcopy(self._groupby)
        clone._orderby = deepcopy(self._orderby)
        clone._queryset = self._queryset
        return clone

    def _aggregate(self, *groupby, **aggregates):
        clone = self._clone()
        clone._aggregates.update(aggregates)
        if groupby:
            clone._groupby = groupby
        return clone

    def apply(self, queryset):
        clone = self._clone()
        clone._queryset = queryset
        return clone

    def filter(self, *values, **filters):
        clone = self._clone()

        raw = []
        args = []
        for v in values:
            if isinstance(v, Q):
                args.append(v)
            else:
                raw.append(v)

        # One or more values allowed values for the applied to this
        # data field. This is currently restricted to an `exact` or `in`
        # operator.
        if raw:
            if len(raw) == 1:
                condition = clone.field_name, raw[0]
            else:
                condition = u'{0}__in'.format(clone.field_name), raw
            clone._filter.append(M(tree=clone.model, **dict([condition])))

        clone._filter.extend(args)

        # Separate out the conditions that apply to aggregations. The
        # non-aggregation conditions will always be applied before the
        # aggregation is applied.
        for key, value in filters.iteritems():
            if key.split(LOOKUP_SEP)[0] in clone._aggregates:
                condition = Q(**dict([(key, value)]))
                clone._having.append(condition)
            else:
                condition = M(tree=clone.model, **dict([(key, value)]))
                clone._filter.append(condition)
        return clone

    def exclude(self, *values, **filters):
        clone = self._clone()

        raw = []
        args = []
        for v in values:
            if isinstance(v, Q):
                args.append(v)
            else:
                raw.append(v)

        # One or more values allowed values for the applied to this
        # data field. This is currently restricted to an `exact` or `in`
        # operator.
        if raw:
            if len(raw) == 1:
                condition = clone.field_name, raw[0]
            else:
                condition = u'{0}__in'.format(clone.field_name), raw
            clone._exclude.append(M(tree=clone.model, **dict([condition])))

        clone._exclude.extend(args)

        # Separate out the conditions that apply to aggregations. The
        # non-aggregation conditions will always be applied before the
        # aggregation is applied.
        for key, value in filters.iteritems():
            if key.split(LOOKUP_SEP)[0] in clone._aggregates:
                condition = ~Q(**dict([(key, value)]))
                clone._having.append(condition)
            else:
                condition = ~M(tree=clone.model, **dict([(key, value)]))
                clone._exclude.append(condition)
        return clone

    def order_by(self, *fields):
        clone = self._clone()
        orderby = []
        for f in fields:
            direction = ''
            if f.startswith('-'):
                f = f[1:]
                direction = '-'
            orderby.append(direction + f)
        clone._orderby = orderby
        return clone

    def groupby(self, *groupby):
        clone = self._aggregate(*groupby)
        return clone

    def count(self, *groupby, **kwargs):
        "Performs a COUNT aggregation."
        distinct = kwargs.get('distinct', False)
        if distinct:
            aggregates = {'distinct_count': Count(self.field_name,
                distinct=distinct)}
        else:
            aggregates = {'count': Count(self.field_name)}
        return self._aggregate(*groupby, **aggregates)

    def sum(self, *groupby):
        "Performs an SUM aggregation."
        aggregates = {'sum': Sum(self.field_name)}
        return self._aggregate(*groupby, **aggregates)

    def avg(self, *groupby):
        "Performs an AVG aggregation."
        aggregates = {'avg': Avg(self.field_name)}
        return self._aggregate(*groupby, **aggregates)

    def min(self, *groupby):
        "Performs an MIN aggregation."
        aggregates = {'min': Min(self.field_name)}
        return self._aggregate(*groupby, **aggregates)

    def max(self, *groupby):
        "Performs an MAX aggregation."
        aggregates = {'max': Max(self.field_name)}
        return self._aggregate(*groupby, **aggregates)

    def stddev(self, *groupby):
        "Performs an STDDEV aggregation."
        aggregates = {'stddev': StdDev(self.field_name)}
        return self._aggregate(*groupby, **aggregates)

    def variance(self, *groupby):
        "Performs an VARIANCE aggregation."
        aggregates = {'variance': Variance(self.field_name)}
        return self._aggregate(*groupby, **aggregates)



from django.db import models
from django.utils.importlib import import_module
from avocado.conf import settings

def import_mixin(app_label):
    if app_label in settings.MIXINS:
        path = settings.MIXINS[app_label].split('.')
        mixin_name = path.pop(-1)
        module = import_module('.'.join(path))
        Mixin = getattr(module, mixin_name)
    else:
        Mixin = models.Model

    return Mixin

def create_mixin(name, module, bases=None, attrs=None, meta=None):
    "Creates an abstract model class based on the provided information."
    bases = bases or (models.Model,)
    attrs = attrs or {}
    meta = meta or {}

    meta.setdefault('abstract', True)

    class Meta(object):
        pass

    # Update Meta with any options that were provided
    if meta:
        for key, value in meta.iteritems():
            setattr(Meta, key, value)

    # Create the initial set of model attributes by copying the fields
    attrs = attrs.copy()
    # Set up a dictionary to simulate declarations within a class
    attrs.update({'__module__': module, 'Meta': Meta})

    # create the class, which automatically triggers ModelBase processing
    return type(name, bases, attrs)

def create_model(name, module, bases=None, fields=None, options=None):
    "Creates a non-abstract model class based on the provided information."
    options = options or {}
    options['abstract'] = False

    return create_mixin(name, module, bases=bases, fields=fields,
        options=options)



import re
import os
import logging
from optparse import make_option
from django.db import DEFAULT_DB_ALIAS
from django.core.exceptions import ImproperlyConfigured
from django.core import management
from django.core.management.base import BaseCommand, CommandError
from south import migration
from avocado.conf import settings
from avocado.core import backup

METADATA_FIXTURE_SUFFIX = settings.METADATA_FIXTURE_SUFFIX
METADATA_MIGRATION_SUFFIX = settings.METADATA_MIGRATION_SUFFIX

METADATA_MIGRATION_TEMPLATE = u"""\
# -*- coding: utf-8 -*-
from south.v2 import DataMigration

class Migration(DataMigration):

    def forwards(self, orm):
        "Perform a 'safe' load using Avocado's backup utilities."
        from avocado.core import backup
        backup.safe_load({fixture_name}, backup_path={backup_path},
            using={using})

    def backwards(self, orm):
        "No backwards migration applicable."
        pass
"""

log = logging.getLogger(__name__)

class Command(BaseCommand):
    help = 'Simple utility for dumping Avocado metadata.'

    option_list = BaseCommand.option_list + (
        make_option('--database', action='store', dest='database',
            default=DEFAULT_DB_ALIAS, help='Nominates a specific database to dump '
                'fixtures from. Defaults to the "default" database.'),
        make_option('--backup-path', action='store', dest='backup_path',
            help='Define a non-temporary path for the migration backup.'),
        make_option('--no-fake', action='store_false',
            help='Prevents the new migration from being immediately faked in the database.')
    )

    def handle(self, migration_suffix=METADATA_MIGRATION_SUFFIX, fixture_suffix=METADATA_FIXTURE_SUFFIX, **options):
        database = options.get('database')
        verbosity = options.get('verbosity')
        backup_path = options.get('backup_path')
        no_fake = options.get('no_fake')

        try:
            fixture_dir = backup.get_fixture_dir()
        except ImproperlyConfigured, e:
            raise CommandError(e.message)

        if not os.path.exists(fixture_dir):
            os.makedir(fixture_dir)
            log.info(u'Created fixture directory: {0}'.format(fixture_dir))
        elif not os.path.isdir(fixture_dir):
            raise CommandError(u'The metadata fixture directory {0}.. is not a directory.'.format(fixture_dir))

        # Only allow valid names
        if re.search('[^_\w]', migration_suffix):
            raise CommandError('Migration names should contain only alphanumeric characters and underscores.')

        fixture_name = backup.next_fixture_name(fixture_suffix, fixture_dir)
        backup.create_fixture(fixture_name, using=database)

        if verbosity > 1:
            log.info(u'Created fixture {0}'.format(fixture_name))

        # Get the Migrations for this app (creating the migrations dir if needed)
        migrations = migration.Migrations(settings.METADATA_MIGRATION_APP,
            force_creation=True, verbose_creation=verbosity > 0)

        # See what filename is next in line. We assume they use numbers.
        next_filename = migrations.next_filename(migration_suffix)
 
        file_contents = METADATA_MIGRATION_TEMPLATE.format(fixture_name=repr(fixture_name),
            backup_path=(backup_path and repr(backup_path) or backup_path), using=repr(database))

        fp = open(os.path.join(migrations.migrations_dir(), next_filename), 'w')
        fp.write(file_contents)
        fp.close()
        log.info(u'Created migration {0}'.format(next_filename))

        if not no_fake:
            # Clear and reload migrations..
            [migrations.pop() for _ in xrange(len(migrations))]
            migrations._load_migrations_module(migrations.application.migrations)

            # Fake migrations up to the current created one..
            management.call_command('migrate', settings.METADATA_MIGRATION_APP,
                os.path.splitext(next_filename)[0], fake=True, database=database)

            log.info('Faked migrations up to the current one')



from zipfile import ZipFile
from cStringIO import StringIO
from string import punctuation
from django.template import Context
from django.template.loader import get_template
from _base import BaseExporter
from _csv import CSVExporter


class SASExporter(BaseExporter):
    short_name = 'SAS'
    long_name = 'Statistical Analysis System (SAS)'

    file_extension = 'zip'
    content_type = 'application/zip'

    preferred_formats = ('sas', 'coded', 'number', 'string')

    num_lg_names = 0

    # informat/format mapping for all datatypes except strings
    sas_informat_map = {
        'key': 'best32.',
        'number': 'best32.',
        'date': 'MMDDYYw.',
        'boolean': 'best32.',
        'datetime': 'DATETIMEw.d',
        'time': 'TIMEw.d'
    }

    sas_format_map = {
        'key': 'best12.',
        'number': 'best12.',
        'date': 'MMDDYYw.',
        'boolean': 'best12.',
        'datatime': 'DATETIMEw.d',
        'time': 'TIMEw.d'
    }

    def _format_name(self, name):
        punc = punctuation.replace('_', '')
        name = str(name).translate(None, punc)
        name = name.replace(' ', '_')
        if name[0].isdigit():
            name = '_' + name
        if len(name) < 30:
            return name
        self.num_lg_names += 1
        sas_name = name[:20]
        sas_name += u'_lg_{0}'.format(self.num_lg_names)
        return sas_name

    def _get_formats(self, sas_name, datafield):
        """This method creates the sas format and informat lists for
        every variable.
        """
        # get the informat/format
        if datafield.simple_type == 'string':
            informat = s_format = u'${0}.'.format(datafield.field.max_length)
        else:
            s_format = self.sas_format_map[datafield.simple_type]
            informat = self.sas_informat_map[datafield.simple_type]

        sas_informat = u'{0:<10}{1:>10}'.format(sas_name, informat)
        sas_format = u'{0:<10}{1:>10}'.format(sas_name, s_format)
        return sas_format, sas_informat

    def _code_values(self, name, field):
        """If field can be coded return the value dictionary
        and the format name for the dictionary
        """
        value_format = u'{0} {0}_f.'.format(name)
        value = u'{0}_f'.format(name)

        codes = []
        for i, (code, label) in enumerate(field.coded_choices):
            codes.append(u'{0}="{1}"'.format(code, label))

        values = u'{0} {1}'.format(value, '\t'.join(codes))
        return value_format, values

    def write(self, iterable, buff=None, template_name='export/script.sas', *args, **kwargs):
        zip_file = ZipFile(self.get_file_obj(buff), 'w')

        formats = []            # sas formats for all fields
        informats = []          # sas informats for all fields
        inputs = []             # field names in sas format
        values = []             # sas value dictionaries
        value_formats = []      # labels for value dictionary
        labels = []             # labels the field names

        for c in self.concepts:
            cfields = c.concept_fields.select_related('datafield')
            for cfield in cfields:
                field = cfield.field
                name = self._format_name(field.field_name)

                # Setting up formats/informats
                format, informat = self._get_formats(name, field)
                formats.append(format)
                informats.append(informat)

                # Add the field names to the input statement
                if field.simple_type == 'string':
                    inputs.append(u'{0} $'.format(name))
                else:
                    inputs.append(name)

                # If a field can be coded create a SAS PROC Format statement
                # that creates a value dictionary
                if field.lexicon:
                    value_format, value = self._code_values(name, field)
                    value_formats.append(value_format)
                    values.append(value)

                # construct labels
                labels.append(u'{0}="{1}"'.format(name, unicode(cfield)))

        data_filename = 'data.csv'
        script_filename = 'script.sas'

        # File buffers
        data_buff = StringIO()
        # Create the data file
        data_exporter = CSVExporter(self.concepts)
        # Overwrite preferred formats for data file
        data_exporter.preferred_formats = self.preferred_formats
        data_exporter.write(iterable, data_buff, *args, **kwargs)

        zip_file.writestr(data_filename, data_buff.getvalue())

        template = get_template(template_name)
        context = Context({
            'data_filename': data_filename,
            'informats': informats,
            'formats': formats,
            'inputs': inputs,
            'labels': labels,
            'values': values,
            'value_formats': value_formats,
        })

        # Write script from template
        zip_file.writestr(script_filename, template.render(context))
        zip_file.close()

        return zip_file



from django import template
from avocado.models import DataField, DataConcept


register = template.Library()


class AvocadoLoadNode(template.Node):
    def __init__(self, model, key, variable):
        self.model = model
        self.key = key
        self.variable = variable

    def render(self, context):
        try:
            if type(self.key) is int:
                obj = self.model.objects.get(pk=self.key)
            else:
                obj = self.model.objects.get_by_natural_key(self.key)
            context[self.variable] = obj
        except self.model.DoesNotExist:
            pass
        return ''


def do_avocado(parser, token):
    """Loads a DataField or DataConcept instance.

    Examples:

        # Get the datafield with a primary key 30
        {% avocado load field 30 as foo %}

        # Get the datafield with a natural key "blog.tag.label"
        {% avocado load field "blog.tag.label" as tag %}

        # Get the dataconcept with a primary key 14
        {% avocado load concept 14 as bar %}
    """

    toks = token.split_contents()

    if len(toks) != 6:
        raise template.TemplateSyntaxError(u'{0} tag requires 6 arguments'.format(toks[0]))

    tag_name, operation, model, key, _as, variable = toks

    if operation != 'load':
        raise template.TemplateSyntaxError(u'{0} {1} is an invalid operation'.format(tag_name, operation))

    if model == 'field':
        model = DataField
    elif model == 'concept':
        model = DataConcept
    else:
        raise template.TemplateSyntaxError(u'{0} ... {1} is not a valid model type'.format(tag_name, model))

    if key.isdigit():
        key = int(key)
    else:
        if model is DataConcept:
            raise template.TemplateSyntaxError(u'{0} ... concept only supports primary keys.'.format(tag_name))
        if key[0] != key[1] and key[0] not in ('"', "'"):
            raise template.TemplateSyntaxError(u'{0} ... {1} natural keys must be quoted'.format(tag_name, key))
        key = key[1:-1]

    return AvocadoLoadNode(model, key, variable)


register.tag('avocado', do_avocado)



from warnings import warn
from avocado.core import loader


class BaseOperatorMetaclass(type):
    def __new__(cls, name, bases, attrs):
        new_cls = type.__new__(cls, name, bases, attrs)
        new_cls.uid = ('-' if new_cls.negated else '') + new_cls.lookup
        return new_cls


class BaseOperator(object):
    __metaclass__ = BaseOperatorMetaclass

    lookup = ''
    short_name = ''
    verbose_name = ''
    negated = False

    def __unicode__(self):
        return u'{0} ({1})'.format(self.verbose_name, self.uid)

    def __repr__(self):
        return u'<Operator: "{0}" ({1})>'.format(self.verbose_name, self.uid)

    @property
    def operator(self):
        warn('self.operator is deprecated, use self.lookup instead',
            DeprecationWarning)
        return self.lookup

    def coerce_to_unicode(self, value):
        return unicode(value)

    def is_valid(self, value):
        raise NotImplemented('Use an Operator subclass')

    def text(self, value):
        raise NotImplemented('Use an Operator subclass')


class SimpleTypeOperator(BaseOperator):
    "Operator class for non-container type values. Includes strings."
    def is_valid(self, value):
        return not hasattr(value, '__iter__')

    def text(self, value):
        value = self.coerce_to_unicode(value)
        return u'{0} {1}'.format(self.verbose_name, value)


class StringOperator(SimpleTypeOperator):
    "Operator class for string-only lookups."
    def is_valid(self, value):
        return isinstance(value, basestring)


class ContainerTypeOperator(BaseOperator):
    "Operator class for container type values. Excludes strings."
    join_string = 'and'
    max_list_size = 3

    def is_valid(self, value):
        return hasattr(value, '__iter__')

    def text(self, value):
        value = map(self.coerce_to_unicode, value)

        last = value[-1]
        length = len(value) - 1

        if length == 0:
            return u'{0} {1}'.format(self.verbose_name, last)

        if length > self.max_list_size:
            head = value[:self.max_list_size]
        else:
            head = value[:-1]
        text = self.verbose_name + ' ' + ', '.join(head)

        # Add the leftover item count for the tail of the list
        tail = length - self.max_list_size
        if tail > 0:
            text += u' ... ({0} more)'.format(tail)

        return text + u' {0} '.format(self.join_string) + last


class Null(BaseOperator):
    lookup = 'isnull'
    short_name = 'is null'
    verbose_name = 'is null'

    def is_valid(self, value):
        return isinstance(value, bool)

    def text(self, value):
        "Do not return value"
        return unicode(self.verbose_name if value else NotNull.verbose_name)


class NotNull(Null):
    short_name = 'not null'
    verbose_name = 'is not null'
    negated = True

    def text(self, value):
        "Do not return value"
        return unicode(self.verbose_name if value else Null.verbose_name)


class Exact(SimpleTypeOperator):
    lookup = 'exact'
    short_name = '='
    verbose_name = 'is'


class NotExact(Exact):
    short_name = '!='
    verbose_name = 'is not'
    negated = True

    def text(self, value):
        # Easier to read 'is False', rather than 'is not True'
        if isinstance(value, bool):
            return u'is {0}'.format(not value)
        return super(NotExact, self).text(value)


# String-specific lookups
class InsensitiveExact(StringOperator):
    lookup = 'iexact'
    short_name = '='
    verbose_name = 'is'


class InsensitiveNotExact(InsensitiveExact):
    short_name = '!='
    verbose_name = 'is not'
    negated = True


class Contains(StringOperator):
    lookup = 'contains'
    short_name = 'contains'
    verbose_name = short_name


class InsensitiveContains(Contains):
    lookup = 'icontains'
    short_name = 'contains'
    verbose_name = short_name


class NotContains(Contains):
    short_name = 'does not contain'
    verbose_name = 'does not contain'
    negated = True


class NotInsensitiveContains(InsensitiveContains):
    short_name = 'does not contain'
    verbose_name = 'does not contain'
    negated = True


# Numerical and lexicographical lookups
class LessThan(SimpleTypeOperator):
    lookup = 'lt'
    short_name = '<'
    verbose_name = 'is less than'


class GreaterThan(SimpleTypeOperator):
    lookup = 'gt'
    short_name = '>'
    verbose_name = 'is greater than'


class LessThanOrEqual(SimpleTypeOperator):
    lookup = 'lte'
    short_name = '<='
    verbose_name = 'is less than or equal to'


class GreaterThanOrEqual(SimpleTypeOperator):
    lookup = 'gte'
    short_name = '>='
    verbose_name = 'is greater than or equal to'


# Operators for container types (excluding strings)
class InList(ContainerTypeOperator):
    lookup = 'in'
    short_name = 'includes'
    verbose_name = 'includes'


class NotInList(InList):
    short_name = 'excludes'
    verbose_name = 'excludes'
    negated = True


class Range(ContainerTypeOperator):
    join_string = 'and'
    lookup = 'range'
    short_name = 'between'
    verbose_name = 'is between'

    def is_valid(self, value):
        return super(Range, self).is_valid(value) and len(value) == 2

    def text(self, value):
        value = map(self.coerce_to_unicode, value)
        return u'{0} {1}'.format(self.verbose_name, ' and '.join(value))


class NotRange(Range):
    short_name = 'not between'
    verbose_name = 'is not between'
    negated = True


# Register operators
registry = loader.Registry()

# General equality
registry.register(Exact, Exact.uid)
registry.register(NotExact, NotExact.uid)

# String operators
registry.register(InsensitiveExact, InsensitiveExact.uid)
registry.register(Contains, Contains.uid)
registry.register(InsensitiveContains, InsensitiveContains.uid)
registry.register(InsensitiveNotExact, InsensitiveNotExact.uid)
registry.register(NotContains, NotContains.uid)
registry.register(NotInsensitiveContains, NotInsensitiveContains.uid)

# Null
registry.register(Null, Null.uid)
registry.register(NotNull, NotNull.uid)

# Numerical or lexicographical comparison
registry.register(LessThan, LessThan.uid)
registry.register(GreaterThan, GreaterThan.uid)
registry.register(LessThanOrEqual, LessThanOrEqual.uid)
registry.register(GreaterThanOrEqual, GreaterThanOrEqual.uid)

# List
registry.register(InList, InList.uid)
registry.register(NotInList, NotInList.uid)

# Range
registry.register(Range, Range.uid)
registry.register(NotRange, NotRange.uid)

loader.autodiscover('operators')



from django.db import models
from .managers import LexiconManager


class Lexicon(models.Model):
    """Abstract class for defining a lexicon.

    To implement, subclass and define the `value` and `label` fields.
    Avocado treats Lexicon subclasses specially since it is such a
    common practice to have tables that represent a lexicon.

    A few of the advantages include:

        - define an arbitrary order of the items
        - define a integer `code` which is useful for downstream clients that
        prefer working with a enumerable set of values such as SAS or R
        - define a verbose/more readable label for each item

    Avocado integrates Lexicon subclasses by using them in the following ways:

        - performing an `init` will ignore the `label`, `code`, and `order`
        fields since they are supplementary to the `value`
        - using a DataField that represents a field on a Lexicon subclass will
        use the `order` field whenever accessing values or applying it to
        a query
    """
    code = models.IntegerField(null=True)
    order = models.IntegerField(default=0)

    objects = LexiconManager()

    class Meta(object):
        abstract = True
        ordering = ['order']

    def __unicode__(self):
        return unicode(self.label or self.value)



from django.db import models


def coerce_float(x):
    try:
        return float(x)
    except (TypeError, ValueError):
        return x

key_funcs = {
    'coerce_float': coerce_float,
}


class LexiconManager(models.Manager):
    def reorder(self, cmp=None, key=None):
        if isinstance(key, basestring):
            if key not in key_funcs:
                raise KeyError(u'No key function named {0}'.format(key))
            key = key_funcs[key]
        items = list(self.get_query_set())
        items.sort(cmp=cmp, key=key)
        for i, item in enumerate(items):
            item.order = i
            item.save()



from modeltree.tree import trees
from modeltree.query import ModelTreeQuerySet
from django.core.exceptions import ValidationError


SORT_DIRECTIONS = ('asc', 'desc')


def has_keys(obj, keys):
    "Check the required keys are present in `obj`"
    for key in keys:
        if key not in obj:
            return False
    return True


class Node(object):
    def __init__(self, columns=None, ordering=None, **context):
        self.concept_ids = columns or []
        self.ordering = ordering or []
        self.tree = context.pop('tree', None)
        self.context = context

    def apply(self, queryset=None, include_pk=True):
        tree = trees[self.tree]
        if queryset is None:
            queryset = tree.get_queryset()
        queryset = ModelTreeQuerySet(tree, query=queryset.query)
        if self.concept_ids:
            model_fields = []
            for f in self.fields:
                if f.lexicon:
                    model_fields.append(f.model._meta.get_field('label'))
                else:
                    model_fields.append(f.field)
            queryset = queryset.select(*model_fields, include_pk=include_pk)
        if self.ordering:
            queryset = queryset.order_by(*self.order_by)
        return queryset

    @property
    def columns(self):
        from avocado.models import DataConcept
        columns = list(DataConcept.objects.filter(pk__in=self.concept_ids))
        columns.sort(key=lambda o: self.concept_ids.index(o.pk))
        return columns

    @property
    def fields(self):
        fields = []
        for concept in self.columns:
            fields.extend(list(concept.fields.select_related('concept_fields')\
                .order_by('concept_fields__order')))
        return fields

    @property
    def order_by(self):
        order_by = []
        if self.ordering:
            from avocado.models import DataConcept
            tree = trees[self.tree]
            ids, directions = zip(*self.ordering)

            columns = list(DataConcept.objects.filter(pk__in=ids))
            columns.sort(key=lambda o: ids.index(o.pk))

            fields = []
            for concept in columns:
                fields.append(list(concept.fields.select_related('concept_fields')\
                    .order_by('concept_fields__order')))

            for i, direction in enumerate(directions):
                direction = direction.lower()
                for f in fields[i]:
                    # Special case for Lexicon-based models, order by their
                    # corresponding `order` field.
                    if f.lexicon:
                        field = f.model._meta.get_field('order')
                        qstr = tree.query_string_for_field(field)
                    else:
                        qstr = tree.query_string_for_field(f.field)
                    if direction == 'desc':
                        qstr = '-' + qstr
                    order_by.append(qstr)
        return order_by


def validate(attrs, **context):
    if not attrs:
        return

    from avocado.models import DataConcept

    columns = attrs.get('columns', [])
    ordering = attrs.get('ordering', [])

    if columns:
        if len(set(columns)) != DataConcept.objects.filter(pk__in=columns).count():
            raise ValidationError('One or more concepts do not exist')

    if ordering:
        ids, directions = zip(*ordering)

        for _dir in directions:
            if _dir.lower() not in SORT_DIRECTIONS:
                raise ValidationError('Invalid sort direction')

        if len(set(ids)) != DataConcept.objects.filter(pk__in=ids).count():
            raise ValidationError('One or more concepts do not exist')


def parse(attrs, **context):
    if not attrs:
        return Node(**context)

    columns = attrs.get('columns', None)
    ordering = attrs.get('ordering', None)
    return Node(columns, ordering, **context)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'DataField.internal'
        db.add_column('avocado_datafield', 'internal',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)

        # Adding field 'DataConcept.ident'
        db.add_column('avocado_dataconcept', 'ident',
                      self.gf('django.db.models.fields.CharField')(max_length=100, null=True),
                      keep_default=False)

        # Adding field 'DataConcept.internal'
        db.add_column('avocado_dataconcept', 'internal',
                      self.gf('django.db.models.fields.BooleanField')(default=False),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'DataField.internal'
        db.delete_column('avocado_datafield', 'internal')

        # Deleting field 'DataConcept.ident'
        db.delete_column('avocado_dataconcept', 'ident')

        # Deleting field 'DataConcept.internal'
        db.delete_column('avocado_dataconcept', 'internal')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'avocado.datacategory': {
            'Meta': {'ordering': "('order', 'name')", 'object_name': 'DataCategory'},
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_column': "'_order'", 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['avocado.DataCategory']"}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'avocado.dataconcept': {
            'Meta': {'ordering': "('order',)", 'object_name': 'DataConcept'},
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['avocado.DataCategory']", 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'fields': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'concepts'", 'symmetrical': 'False', 'through': "orm['avocado.DataConceptField']", 'to': "orm['avocado.DataField']"}),
            'formatter_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'concepts+'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ident': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'}),
            'internal': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name_plural': ('django.db.models.fields.CharField', [], {'max_length': '60', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_column': "'_order'", 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'queryview': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'concepts+'", 'blank': 'True', 'to': "orm['sites.Site']"})
        },
        'avocado.dataconceptfield': {
            'Meta': {'ordering': "('order',)", 'object_name': 'DataConceptField'},
            'concept': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'concept_fields'", 'to': "orm['avocado.DataConcept']"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'field': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'concept_fields'", 'to': "orm['avocado.DataField']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'name_plural': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_column': "'_order'", 'blank': 'True'})
        },
        'avocado.datacontext': {
            'Meta': {'object_name': 'DataContext'},
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'composite': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'db_column': "'_count'"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'json': ('jsonfield.fields.JSONField', [], {'default': '{}', 'null': 'True', 'blank': 'True'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'session': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'datacontext+'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'avocado.datafield': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_name', 'model_name', 'field_name'),)", 'object_name': 'DataField'},
            'app_name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['avocado.DataCategory']", 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'data_modified': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'data_source': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'enumerable': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'field_name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'fields+'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'internal': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'model_name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name_plural': ('django.db.models.fields.CharField', [], {'max_length': '60', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_column': "'_order'", 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'searchable': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'fields+'", 'blank': 'True', 'to': "orm['sites.Site']"}),
            'translator': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'unit': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'unit_plural': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'})
        },
        'avocado.dataview': {
            'Meta': {'object_name': 'DataView'},
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'db_column': "'_count'"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'json': ('jsonfield.fields.JSONField', [], {'default': '{}', 'null': 'True', 'blank': 'True'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'session': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'dataview+'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['avocado']


from django.db import models
from django.db.models import Q
from django.db import transaction
from django.conf import settings
from django.db.models.manager import ManagerDescriptor
from django.core.exceptions import ImproperlyConfigured
from avocado.conf import OPTIONAL_DEPS, requires_dep
from avocado.core.managers import PublishedManager, PublishedQuerySet


class DataFieldQuerySet(PublishedQuerySet):
    def published(self, user=None, perm='avocado.view_datafield'):
        """Fields can be restricted to one or more sites, so the published
        method is extended to support filtering by site.
        """
        published = super(DataFieldQuerySet, self).published()

        # Remove internal
        published = published.exclude(internal=True)

        # All published concepts associated with the current site
        # (or no site)
        sites = Q(sites=None) | Q(sites__id=settings.SITE_ID)
        published = published.filter(sites)

        if user:
            if not OPTIONAL_DEPS['guardian']:
                raise ImproperlyConfigured('django-guardian must installed ' \
                    'for object-level permissions.')
            from guardian.shortcuts import get_objects_for_user
            published = get_objects_for_user(user, perm, published)
        return published.distinct()


class DataConceptQuerySet(PublishedQuerySet):
    def published(self, user=None, perm='avocado.view_datafield'):
        """Concepts can be restricted to one or more sites, so the published
        method is extended to support filtering by site. In addition, concepts
        should not be visible if their associated fields are not all available.
        """
        published = super(DataConceptQuerySet, self).published()

        # Remove internal
        published = published.exclude(internal=True)

        # All published concepts associated with the current site
        # (or no site)
        sites = Q(sites=None) | Q(sites__id=settings.SITE_ID)
        published = published.filter(sites)

        # Concepts that contain at least one unpublished or archived datafield
        # are removed from the set to prevent exposing unprepared data
        from avocado.models import DataField
        fields_q = Q(archived=True) | Q(published=False)
        if user:
            if not OPTIONAL_DEPS['guardian']:
                raise ImproperlyConfigured('django-guardian must installed ' \
                    'for object-level permissions.')
            from guardian.shortcuts import get_objects_for_user
            # If a user is specified, they must also have a permission for
            # accessing the data fields. All data fields that the user does
            # NOT have access to must also be removed from the set.
            restricted_fields = DataField.objects\
                .exclude(pk__in=get_objects_for_user(user, perm))
            fields_q = fields_q | Q(pk__in=restricted_fields)

        shadowed = DataField.objects.filter(fields_q)
        concepts = published.exclude(fields__in=shadowed).distinct()
        return concepts


class DataFieldManagerDescriptor(ManagerDescriptor):
    """Manager descriptor customized to allow for `f.objects` which returns
    a QuerySet of the underlying model objects which DataField represents.
    """
    def __init__(self, manager):
        self.manager = manager

    def __get__(self, instance, type=None):
        if instance and isinstance(instance.field, models.AutoField):
            return instance.model.objects.all()
        return super(DataFieldManagerDescriptor, self).__get__(instance, type)


class DataFieldManager(PublishedManager):
    "Manager for the `DataField` model."

    def contribute_to_class(self, model, name):
        "Override to use custom manager descriptor."
        super(DataFieldManager, self).contribute_to_class(model, name)
        setattr(model, name, DataFieldManagerDescriptor(self))

    def get_query_set(self):
        return DataFieldQuerySet(self.model, using=self._db)

    def get_by_natural_key(self, app_name, model_name=None, field_name=None):
        queryset = self.get_query_set()
        if type(app_name) is int:
            datafield = queryset.get(id=app_name)
        else:
            keys = ['app_name', 'model_name', 'field_name']
            if type(app_name) is list:
                values = app_name
            elif type(app_name) and '.' in app_name:
                values = app_name.split('.')
            else:
                values = [app_name, model_name, field_name]
            datafield = queryset.get(**dict(zip(keys, values)))
        return datafield

    @requires_dep('haystack')
    def search(self, content, queryset=None, max_results=10):
        from haystack.query import RelatedSearchQuerySet
        sqs = RelatedSearchQuerySet().models(self.model).load_all()\
            .auto_query(content)
        if queryset is not None:
            sqs = sqs.load_all_queryset(self.model, queryset)
        if max_results:
            return sqs[:max_results]
        return sqs


class DataConceptManager(PublishedManager):
    "Manager for the `DataConcept` model."
    def get_query_set(self):
        return DataConceptQuerySet(self.model, using=self._db)

    @requires_dep('haystack')
    def search(self, content, queryset=None, max_results=10):
        from haystack.query import RelatedSearchQuerySet
        sqs = RelatedSearchQuerySet().models(self.model).load_all()\
            .auto_query(content)
        if queryset is not None:
            sqs = sqs.load_all_queryset(self.model, queryset)
        if max_results:
            return sqs[:max_results]
        return sqs

    @transaction.commit_on_success
    def create_from_field(self, datafield, save=True, **kwargs):
        """Derives a DataConcept from this DataField's descriptors. Additional
        keyword arguments can be passed in to customize the new DataConcept
        object. The DataConcept can also be optionally saved by setting the
        `save` flag.
        """
        for key, value, in datafield.descriptors.iteritems():
            kwargs.setdefault(key, value)

        concept = self.model(**kwargs)

        if save:
            from avocado.models import DataConceptField
            concept.save()
            cfield = DataConceptField(field=datafield, concept=concept)
            concept.concept_fields.add(cfield)
        return concept


class DataCategoryManager(PublishedManager):
    "Manager for the `DataCategory` model."



import jsonfield
from django.db import models
from modeltree.tree import trees
from . import parsers


class AbstractDataContext(models.Model):
    """JSON object representing one or more data field conditions. The data may
    be a single condition, an array of conditions or a tree stucture.

    This corresponds to the `WHERE` statements in a SQL query.
    """
    json = jsonfield.JSONField(null=True, blank=True, default=dict,
        validators=[parsers.datacontext.validate])
    composite = models.BooleanField(default=False)
    count = models.IntegerField(null=True, db_column='_count')

    class Meta(object):
        abstract = True

    def _combine(self, other, operator):
        if not isinstance(other, self.__class__):
            raise TypeError('Other object must be a DataContext instance')
        cxt = self.__class__(composite=True)
        cxt.user_id = self.user_id or other.user_id
        if self.json and other.json:
            cxt.json = {
                'type': operator,
                'children': [
                    {'id': self.pk, 'composite': True},
                    {'id': other.pk, 'composite': True}
                ]
            }
        elif self.json:
            cxt.json = {'id': self.pk, 'composite': True}
        elif other.json:
            cxt.json = {'id': other.pk, 'composite': True}
        return cxt

    def __and__(self, other):
        return self._combine(other, 'and')

    def __or__(self, other):
        return self._combine(other, 'or')

    @property
    def model(self):
        "The model this context represents with respect to the count."
        if self.count is not None:
            return trees.default.root_model

    @classmethod
    def validate(cls, attrs, **context):
        "Validate `attrs` as a context."
        parsers.datacontext.validate(attrs, **context)

    def parse(self, tree=None, **context):
        "Returns a parsed node for this context."
        return parsers.datacontext.parse(self.json, tree=tree, **context)

    def apply(self, queryset=None, tree=None, **context):
        "Applies this context to a QuerySet."
        if tree is None and queryset is not None:
            tree = queryset.model
        return self.parse(tree=tree, **context).apply(queryset=queryset)

    def language(self, tree=None, **context):
        return self.parse(tree=tree, **context).language


class AbstractDataView(models.Model):
    """JSON object representing one or more data field conditions. The data may
    be a single condition, an array of conditions or a tree stucture.

    This corresponds to the `SELECT` and `ORDER BY` statements in a SQL query.
    """
    json = jsonfield.JSONField(null=True, blank=True, default=dict,
        validators=[parsers.dataview.validate])
    count = models.IntegerField(null=True, db_column='_count')

    class Meta(object):
        abstract = True

    @classmethod
    def validate(cls, attrs, **context):
        "Validates `attrs` as a view."
        parsers.dataview.validate(attrs, **context)

    def parse(self, tree=None, **context):
        "Returns a parsed node for this view."
        return parsers.dataview.parse(self.json, tree=tree, **context)

    def apply(self, queryset=None, tree=None, include_pk=True, **context):
        "Applies this context to a QuerySet."
        if tree is None and queryset is not None:
            tree = queryset.model
        return self.parse(tree=tree, **context).apply(queryset=queryset,
            include_pk=include_pk)



from modeltree.tree import trees
from django.core.exceptions import ValidationError

AND = 'AND'
OR = 'OR'
BRANCH_KEYS = ('children', 'type')
CONDITION_KEYS = ('id', 'value')
COMPOSITE_KEYS = ('id', 'composite')
LOGICAL_OPERATORS = ('and', 'or')


def has_keys(obj, keys):
    "Check the required keys are present in `obj`"
    for key in keys:
        if key not in obj:
            return False
    return True


def is_branch(obj):
    "Validates required structure for a branch node"
    if has_keys(obj, keys=BRANCH_KEYS):
        if obj['type'] not in LOGICAL_OPERATORS:
            raise ValidationError('Invalid branch operator')
        length = len(obj['children'])
        if length < 2:
            raise ValidationError('Branch must contain two or more children')
        return True


def is_condition(obj):
    "Validates required structure for a condition node"
    if has_keys(obj, keys=CONDITION_KEYS):
        return True


def is_composite(obj):
    if has_keys(obj, keys=COMPOSITE_KEYS):
        if obj['composite'] is not True:
            raise ValidationError('Composite key must be set to True.')
        return True


class Node(object):
    condition = None
    annotations = None
    extra = None
    language = None

    def __init__(self, **context):
        self.tree = context.pop('tree', None)
        self.context = context

    def apply(self, queryset=None, distinct=True):
        if queryset is None:
            queryset = trees[self.tree].get_queryset()
        if self.annotations:
            queryset = queryset.values('pk').annotate(**self.annotations)
        if self.condition:
            queryset = queryset.filter(self.condition)
        if self.extra:
            queryset = queryset.extra(**self.extra)
        if distinct:
            queryset = queryset.distinct()
        return queryset


class Condition(Node):
    "Contains information for a single query condition."
    def __init__(self, id, operator, value, **context):
        self.id = id
        self.operator = operator
        self.value = value
        super(Condition, self).__init__(**context)

    @property
    def _meta(self):
        if not hasattr(self, '__meta'):
            self.__meta = self.field.translate(operator=self.operator,
                value=self.value, tree=self.tree, **self.context)
        return self.__meta

    @property
    def field(self):
        if not hasattr(self, '_field'):
            from avocado.models import DataField
            self._field = DataField.objects.get_by_natural_key(self.id)
        return self._field

    @property
    def condition(self):
        return self._meta['query_modifiers'].get('condition', None)

    @property
    def annotations(self):
        return self._meta['query_modifiers'].get('annotations', None)

    @property
    def extra(self):
        return self._meta['query_modifiers'].get('extra', None)

    @property
    def language(self):
        meta = self._meta.copy()
        meta.pop('query_modifiers')
        cleaned = meta.pop('cleaned_data')
        meta['language'] = cleaned['language']
        return meta


class Branch(Node):
    "Provides a logical relationship between it's children."
    def __init__(self, type, **context):
        self.type = (type.upper() == AND) and AND or OR
        self.children = []
        super(Branch, self).__init__(**context)

    def _combine(self, q1, q2):
        if self.type.upper() == OR:
            return q1 | q2
        return q1 & q2

    @property
    def condition(self):
        if not hasattr(self, '_condition'):
            condition = None
            for node in self.children:
                if node.condition:
                    if condition:
                        condition = self._combine(node.condition, condition)
                    else:
                        condition = node.condition
            self._condition = condition
        return self._condition

    @property
    def annotations(self):
        if not hasattr(self, '_annotations'):
            self._annotations = {}
            for node in self.children:
                if node.annotations:
                    self._annotations.update(node.annotations)
        return self._annotations

    @property
    def extra(self):
        extra = {}
        for node in self.children:
            if not node.extra:
                continue
            for key, value in node.extra.items():
                _type = type(value)
                # Initialize an empty container for the value type..
                extra.setdefault(key, _type())
                if _type is list:
                    current = extra[key][:]
                    [extra[key].append(x) for x in value if x not in current]
                elif _type is dict:
                    extra[key].update(value)
                else:
                    raise TypeError('The `.extra()` method only takes list of dicts as keyword values')
        return extra

    @property
    def language(self):
        out = {'type': self.type.lower(), 'children': []}
        for node in self.children:
            out['children'].append(node.language)
        return out


def validate(attrs, **context):
    if type(attrs) is not dict:
        raise ValidationError('Object must be of type dict')
    if not attrs:
        return
    if is_composite(attrs):
        from avocado.models import DataContext
        try:
            if 'user' in context:
                cxt = DataContext.objects.get(id=attrs['id'], user=context['user'])
            else:
                cxt = DataContext.objects.get(id=attrs['id'])
        except DataContext.DoesNotExist:
            raise ValidationError(u'DataContext "{0}" does not exist.'.format(attrs['id']))
        validate(cxt.json, **context)
    elif is_condition(attrs):
        from avocado.models import DataField
        try:
            field = DataField.objects.get_by_natural_key(attrs['id'])
        except DataField.DoesNotExist, e:
            raise ValidationError(e.message)
        field.validate(operator=attrs['operator'], value=attrs['value'])
    elif is_branch(attrs):
        map(lambda x: validate(x), attrs['children'])
    else:
        raise ValidationError(u'Object neither a branch nor condition: {0}'.format(attrs))


def parse(attrs, **context):
    if not attrs:
        node = Node(**context)
    elif is_composite(attrs):
        from avocado.models import DataContext
        if 'user' in context:
            cxt = DataContext.objects.get(id=attrs['id'], user=context['user'])
        else:
            cxt = DataContext.objects.get(id=attrs['id'])
        return parse(cxt.json, **context)
    elif is_condition(attrs):
        node = Condition(attrs['id'], attrs.get('operator', None), attrs['value'], **context)
    else:
        node = Branch(attrs['type'], **context)
        node.children = map(lambda x: parse(x, **context), attrs['children'])
    return node



from datetime import datetime
from optparse import make_option
from django.db.models import Q
from django.core.management.base import BaseCommand
from avocado.models import DataField


class Command(BaseCommand):
    """
    SYNOPSIS::

        python manage.py avocado data [options...] labels...

    DESCRIPTION:

        Finds all models referenced by the app, model or field `labels` and
        updates data-related properties such as caches and pre-calculated
        values.

    OPTIONS:

        `--modified` - Updates the `data_modified` on `DataField` instances
        corresponding the labels. This is primarily used for cache
        invalidation.
    """

    help = '\n'.join([
        'Finds all models referenced by the app, model or field `labels` and',
        'updates data-related properties such as caches and pre-calculated',
        'values.',
    ])

    args = 'app [app.model, [app.model.field, [...]]]'

    option_list = BaseCommand.option_list + (
        make_option('-m', '--modified', action='store_true',
            dest='update_data_modified', default=False,
            help='Update `data_modified` timestamp on `DataField` instances'),
    )

    def handle(self, *args, **options):
        "Handles app_label or app_label.model_label formats."

        update_data_modified = options['update_data_modified']

        if not update_data_modified:
            print 'Nothing to do.'
            return

        conditions = []

        for label in args:
            labels = label.split('.')

            # Specific field
            if len(labels) == 3:
                app, model, field = labels
                conditions.append(Q(app_name=app, model_name=model, field_name=field))
            # All fields for a model
            elif len(labels) == 2:
                app, model = labels
                conditions.append(Q(app_name=app, model_name=model))
            # All fields for each model in the app
            else:
                app = labels[0]
                conditions.append(Q(app_name=app))

        fields = DataField.objects.all()
        if conditions:
            q = Q()
            for x in conditions:
                q = q | x
            fields = DataField.objects.filter(q)
        updated = fields.update(data_modified=datetime.now())
        print u'{0} DataFields have been updated'.format(updated)



from avocado.core import loader


class QueryView(object):
    """Provides a mean of defining/deriving additional metadata from a
    DataConcept and/or it's associated DataField for use downstream by a
    client application. A _default_ QueryView reference can defined for
    `DataConcept` via it's `queryview` field.

    To implement, override the `process` method and return a `dict` that can
    be serialized downstream.

    Additional context _may_ be supplied (but not guaranteed to be supplied)
    via the `context` dict such as the `request` object while be processed
    during an HTTP request/response cycle.

    `QueryView`s are not client specific. It is assumed that clients can make
    use of as little or as much metadata as they choose. That being said, if
    multiple clients are being used, it may be important to namespace the
    relevant metadata so to not cause key collisions.
    """
    def __call__(self, concept, **context):
        return self.process(concept, **context)

    def __unicode__(self):
        return u'{0}'.format(self.name)

    def process(self, concept, **context):
        return {}


registry = loader.Registry(default=QueryView)
loader.autodiscover('queryviews')


import inspect
from django.core.serializers.json import DjangoJSONEncoder
from _base import BaseExporter


class JSONGeneratorEncoder(DjangoJSONEncoder):
    "Handle generator objects and expressions."
    def default(self, obj):
        if inspect.isgenerator(obj):
            return list(obj)
        return super(JSONGeneratorEncoder, self).default(obj)


class JSONExporter(BaseExporter):
    short_name = 'JSON'
    long_name = 'JavaScript Object Notation (JSON)'

    file_extension = 'json'
    content_type = 'application/json'

    preferred_formats = ('json', 'number', 'string')

    def write(self, iterable, buff=None, *args, **kwargs):
        buff = self.get_file_obj(buff)

        encoder = JSONGeneratorEncoder()
        for chunk in encoder.iterencode(self.read(iterable, *args, **kwargs)):
            buff.write(chunk)
        return buff
